/*********************************************************************
 * webcam_server.c 0.30                                              *
 *                                                                   *
 * (c) 2002 Donn Morrison dmorriso@gulf.uvic.ca                      *
 *                                                                   *
 * code used from Gerd Knorr's xawtv (libng)                         *
 * - and -                                                           *
 * Cory Lueninghoener's gqcam                                        *
 *                                                                   *
 *    waits for connections from a viewer and sends                  *
 *    jpeg encoded captures as a live video feed                     *
 *                                                                   *
 *********************************************************************/

#ifndef _WEBCAM_SERVER_H_INCLUDED_
#define _WEBCAM_SERVER_H_INCLUDED_

#include <pthread.h>
#include "grab-ng.h"

pthread_mutex_t lock_id;
pthread_mutex_t lock_info;
pthread_mutex_t lock_queue;
pthread_mutex_t lock_log;
sem_t sem_con;

struct imagequeue
{
	int id;
	int ref;
	pthread_mutex_t lock_ref;
	char *jpeg_data;
	long len;
	struct imagequeue *next;
	struct imagequeue *prev;
};

struct options
{
	int port;
	int verbose;
	int daemon;
	int retry_init;
	char *caption;
	char *devfile;
	int jpeg_quality;
	int width;
	int height;
	int swap_rgb;
	int test_fps;
	int use_ng;
	int flip_horiz;
	int flip_vert;
	int rotate;
	int gamma;
	int text_fg;
	int text_bg;
	char *logfile;
};

struct status
{
	int num_users;
	int id_newest_img;
};

struct caminfo
{
	int dev;
	FILE *log;
	int grab_thread_alive;
	pthread_t pt_grab;

	char *raw_image;

	struct options o;
	struct status s;
	struct imagequeue *queue;

	struct video_capability vid_caps;
	struct video_window vid_win;
	struct video_picture vid_pic;
	
	/* libng */
	const struct ng_vid_driver *ng_drv;
	void *ng_h_drv;
	struct ng_video_fmt ng_fmt,ng_gfmt;
	struct ng_video_conv *ng_conv;
	void *ng_hconv;
};

struct connection
{
	int socketfd;
	struct sockaddr_in remote_addr;
	u_long total_bytes;
	u_long total_frames;
	u_long id_last;
	int old_images;
	
	struct caminfo *cam;
	
	time_t start;
	time_t end;
};


struct imagequeue *peek(struct caminfo *cam);
int clean(struct caminfo *cam);
void push(struct caminfo *cam, struct imagequeue *new);

static int grab_thread(struct caminfo *cam);
static int watchdog_thread(struct caminfo *cam);

int open_cam(struct caminfo *cam, char *devfile);
int close_cam(struct caminfo *cam);
int set_cam_info(struct caminfo *cam);
void get_cam_info(struct caminfo *cam);
int read_timeout(int fd, char *dest, int size, int seconds);

int ng_grab_init(struct caminfo *cam);
char* ng_grab_one(struct caminfo *cam, int *width, int *height);

void usage();
int parse_args(struct caminfo *cam, int argc, char *argv[]);
void log(struct caminfo *cam, char *text);

int create_and_listen(int port);
static int handle_connection(struct connection *con);

static unsigned char *
rotate_image(unsigned char * in, int *wp, int *hp, int rot,
	     int top, int left, int bottom, int right);
static unsigned int
compare_images(unsigned char *last, unsigned char *current,
	       int width, int height);
void fliph(char *image, int width, int height);
void flipv(char *image, int width, int height);
void swap_rgb24(char *mem, int n);
void adjust_gamma(char *mem, int n, int corr);
void add_text(struct caminfo *cam, char *image, int width, int height);
int raw_to_jpeg(char *raw_image, char **jpeg_data, int w, int h, int quality);

#endif
